﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class CodeGrid
{
    public event System.Action OnSizeChanged;

    public int[] Codes { get; private set; }
    

    public int Rows
    {
        get { return rows; }
        set { value = Mathf.Max(value, 1); if (rows != value) { int r = rows, c = cols; rows = value; rebuildCodeArray(c, r); } }
    }
    public int Cols
    {
        get { return cols; }
        set { value = Mathf.Max(value, 1); if (cols != value) { int r = rows, c = cols; cols = value; rebuildCodeArray(c, r); } }
    }

    public int Get(int col, int row) { return Codes[col + row * Cols]; }
    public void Set(int col, int row, int value) { Codes[col + row * Cols] = value; }

    int cols;
    int rows;

    public CodeGrid(int cols, int rows)
    {
        Cols = cols;
        Rows = rows;
    }

    public CodeGrid(CodeGrid copySource)
    {
        Cols = copySource.Cols;
        Rows = copySource.Rows;
        for (int i = 0; i < Codes.Length; i++) Codes[i] = copySource.Codes[i];
    }

    public void Trim(bool trimCols, bool trimRows)
    {
        if (!trimRows && !trimCols)
            return; // dummy, set one to true at least!

        int newCols = trimCols ? 0 : Cols, newRows = trimRows ? 0 : Rows;
        
        for (int x = 0; x < Cols; x++)
            for (int y = 0; y < Rows; y++)
                if (Get(x, y) != 0)
                {
                    newCols = Mathf.Max(newCols, x + 1);
                    newRows = Mathf.Max(newRows, y + 1);
                }

        Cols = newCols;
        Rows = newRows;
    }

    void rebuildCodeArray(int prevCols, int prevRows)
    {
        int[] newCodes = new int[Rows * Cols];

        if (Codes != null) // preserve existing chars 
        {
            for (int x = 0; x < Cols && x < prevCols; x++)
                for (int y = 0; y < Rows && y < prevRows; y++)
                    newCodes[x + y * Cols] = Codes[x + y * prevCols];
        }

        Codes = newCodes;

        if (OnSizeChanged != null)
            OnSizeChanged();
    }
}
