﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class InputMessage : Message
{

    public event System.Action<CodeGrid> OnSend;

    public int CursorIndex;
    public RectTransform CursorImage;

    public AudioSource PlayOnKeyTap;

    const int MaxRows = 4;

    public void Send()
    {
        if(OnSend != null)
        {
            CodeGrid newGrid = new CodeGrid(Codes);
            newGrid.Trim(false, true);
            OnSend(newGrid);
        }

        //erase message
        Codes.Rows = 1;
        for (int i = 0; i < Codes.Codes.Length; i++) Codes.Codes[i] = 0;
        CursorIndex = 0;
        refreshImages();
    }

    public void EnterCode(int code)
    {
        if (PlayOnKeyTap != null) PlayOnKeyTap.Play();

        if (CursorIndex >= MaxCols * MaxRows)
            return;

        if (CursorIndex < Codes.Codes.Length)
        {
            if (Codes.Codes[Codes.Codes.Length - 1] != 0 && Codes.Rows < MaxRows)
                Codes.Rows++;

            for (int i = Codes.Codes.Length - 1; i > CursorIndex; i--)
            {
                Codes.Codes[i] = Codes.Codes[i - 1];
            }
        }
        else
        {
            while (CursorIndex >= Codes.Codes.Length)
                if (Codes.Cols < MaxCols) Codes.Cols++;
                else Codes.Rows++;
        }

        Codes.Codes[CursorIndex] = code;
        CursorIndex = Mathf.Min(CursorIndex + 1, MaxCols * MaxRows);

        refreshImages();
    }

    public void EnterBackspace()
    {
        if (PlayOnKeyTap != null) PlayOnKeyTap.Play();

        if (CursorIndex > 0)
        {
            CursorIndex--;
            for (int i = CursorIndex; i < Codes.Codes.Length - 1; i++)
                Codes.Codes[i] = Codes.Codes[i + 1];
            Codes.Codes[Codes.Codes.Length - 1] = 0;
        }

        for (int i = Codes.Rows - 1; i > 0; i--)
        {
            if (GetVisibleCols(i) == 0)
                Codes.Rows--;
        }

        refreshImages();
    }

    public void EnterReturn()
    {
        if (PlayOnKeyTap != null) PlayOnKeyTap.Play();

        int cursorCol = CursorIndex % MaxCols, cursorRow = CursorIndex / MaxCols;

        if (cursorRow >= Codes.Rows - 1)
        {
            if (Codes.Rows >= 4)
                return;
            Codes.Rows++;
        }

        for (int i = cursorCol; i < MaxCols; i++)
        {
            float vol = PlayOnKeyTap.volume;
            PlayOnKeyTap.volume = 0;
            EnterCode(0);
            PlayOnKeyTap.volume = vol;
        }
        CursorIndex = (cursorRow + 1) * MaxCols;
    }


    // Use this for initialization
    void Start()
    {
        var inputMoveCursor = Background.GetComponent<InputMoveCursor>();
        inputMoveCursor.OnMoveCursor += InputMoveCursor_OnMoveCursor;

        init();
        onResize();
    }

    private void InputMoveCursor_OnMoveCursor(Vector2 curPos)
    {
        int x = (int)((curPos.x + IconSize / 2) / (IconSize + PaddingX));
        int y = (Codes.Rows - 1) - (int)(curPos.y / (IconSize + PaddingY));
        Debug.Log("xy=" + x + "," + y);
        CursorIndex = Mathf.Clamp(y * Codes.Cols + x, 0, Codes.Codes.Length);
    }


    // Update is called once per frame
    void Update()
    {
        setCursorPos();

    }

    void setCursorPos()
    {
        if(CursorIndex < images.Count)
        {
            Vector2 imgPos = images[CursorIndex].rectTransform.anchoredPosition;
            CursorImage.anchoredPosition = imgPos + new Vector2(0, 0.5f * PaddingY);
        }
        else
        {
            Vector2 imgPos = images[images.Count - 1].rectTransform.anchoredPosition;
            CursorImage.anchoredPosition = imgPos + new Vector2( IconSize, 0.5f * PaddingY);
        }
    }
}
